#!/bin/sh
# the next line restarts using wish \
exec tclsh8.0 "$0" ${1+"$@"}
#

foreach {sourcedir targetdir version libdir bindir} $argv break
cd $sourcedir

set targetdir [file join $targetdir ClassyTk$version]
puts "Building distribution in $targetdir"

proc clean {filemode dirmode dir} {
	if [catch {glob [file join $dir *]} files] return
	foreach file $files {
		if [regexp {~$} $file] {
			file delete $file
		} elseif [regexp {.save$} $file] {
			file delete $file
		} elseif [regexp "[info sharedlibextension]\$" $file] {
		} elseif [file isdirectory $file] {
			catch {file attributes $file -permissions $dirmode}
			clean $filemode $dirmode $file
		} else {
			catch {file attributes $file -permissions $filemode}
		}
	}
}

# Main Program
# ---------------------------------------------------------
	puts "Making target directory"
	file delete -force $targetdir
	file mkdir $targetdir
	file mkdir $bindir
	puts "Copying files"
	file copy apps bin conf dialogs help html_library-0.3 lib patches template widgets $targetdir
	if {"$::tcl_platform(platform)" == "windows"} {
		file copy -force conf/themes/Windows $targetdir/conf/conf
		exec touch $targetdir/conf/conf
	}
	file copy README CHANGES pkgIndex.tcl $targetdir
	catch {file copy [glob build/*.dll] $targetdir}
	catch {file copy [glob build/*.lib] $libdir}
	puts "Checking permissions"
	clean 0644 0755 $targetdir
	catch {file delete [file join $targetdir apps cdraw large.cld]}
	puts "Making links to bin directory"
	foreach file [glob [file join $targetdir bin *]] {
		catch {file attributes $file -permissions 0755}
		set dst [file join $bindir [file tail $file]]
		file delete -force $dst
		exec ln -s $file $dst
	}
	foreach dir [glob [file join $targetdir apps *]] {
		set name [file tail $dir]
		set file [file join $dir $name.tcl]
		set dst [file join $bindir $name]
		puts "Making links to app $name"
		catch {file attributes $file -permissions 0755}
		file delete -force $dst
		exec ln -s $file $dst
	}
	catch {file attributes [file join $targetdir template template.tcl] -permissions 0755}
	puts "done"
exit


