#
# ClassyTk Widgets 
# ----------------- Peter De Rijk
#
# Classy::selectfile
# ----------------------------------------------------------------------
#doc selectfile title {
#selectfile
#} index {
# Dialogs
#} shortdescr {
# select a file to open
#}
#doc {selectfile selectfile} cmd {
#selectfile ?option value ...?
#} descr {
# returns a filename selected by the user. The selection
# method depends om the ClassyTk configuration. Possible options are
#<dl>
#<dt>-defaultextension
#<dt>-filetypes
#<dt>-initialdir
#<dt>-initialfile
#<dt>-title
#<dt>-filter
#<dt>-selectmode<dd>single, browse, multiple, extended or persistent
#<dt>-default
#<dt>-combo {} 20
#<dt>-combopreset {} {}
#<dt>-filtercombo {} 20
#<dt>-filtercombopreset {} {}
#<dt>-help
#</dl>
#}
proc Classy::selectfile {args} {
	global tcl_platform
	Classy::parseopt $args opt {
		-defaultextension {} {}
		-filetypes {} {}
		-initialdir {} {}
		-initialfile {} {}
		-title {} {Select file}
		-filter {} {}
		-selectmode {single browse multiple extended persistent} browse
		-default {} {}
		-combo {} 20
		-combopreset {} {}
		-filtercombo {} 20
		-filtercombopreset {} {}
		-help {} Classy_file_select
	}
	set filter $opt(-filter)
	if {"$filter"==""} {
		set dir [::Classy::Default get app Classy__FileSelect__curfilter]
	}
	if {"$filter"==""} {
		set filter *
	}
	set dir $opt(-initialdir)
	if {"$dir"==""} {
		set dir [::Classy::Default get app Classy__FileSelect__curdir]
	}
	if {"$dir"==""} {
		set dir [pwd]
	}
	set dir [file nativename [lindex $dir 0]]
	if {("$tcl_platform(platform)"=="windows")&&("[option get . selectFile SelectFile]"=="Win")} {
		if {"$opt(-selectmode)"=="single"} {
			set opt(-selectmode) browse
		}
		if {("$opt(-filetypes)" == "")&&("$opt(-defaultextension)" != "")} {
			set opt(-filetypes) [list \
				[list "[string range $opt(-defaultextension) 1 end] file" \
				$opt(-defaultextension)] {{All Files} *}]
		}
		set result [Classy::GetOpenFile -defaultextension $opt(-defaultextension) \
			-filetypes $opt(-filetypes) -initialdir $dir \
			-initialfile $opt(-initialfile) -title $opt(-title) \
			-selectmode $opt(-selectmode)]
		if {"$opt(-selectmode)"=="browse"} {
			Classy::Default set app Classy__FileSelect__curdir [list [file dirname $result]]
			return $result
		} elseif {[llength $result]<2} {
			Classy::Default set app Classy__FileSelect__curdir [list [file dirname [lindex $result 0]]]
			return $result
		} else {
			set dir [list_shift result]
			Classy::Default set app Classy__FileSelect__curdir [list $dir]
			set temp ""
			foreach file $result {
				lappend temp [file join $dir $file]
			}
			return $temp
		}
	} else {
		catch {destroy .classy__.selectfile}
		Classy::FileSelect .classy__.selectfile -dir $dir \
			-title $opt(-title) -textvariable ::Classy::selectfile \
			-filter $filter -selectmode $opt(-selectmode) -help $opt(-help) \
			-default $opt(-default) -combo $opt(-combo) -combopreset $opt(-combopreset) \
			-filtercombo $opt(-filtercombo) -filtercombopreset $opt(-filtercombopreset) \
			-closecommand {set ::Classy::selectfile ""}
		if {"$opt(-initialfile)" != ""} {
			.classy__.selectfile set $opt(-initialfile)
		}
		tkwait window .classy__.selectfile
		return $::Classy::selectfile
	}
}

